@@ -7,21 +7,21 @@
 
 from __future__ import absolute_import
 
+from .pycompat import getattr
 from . import (
     encoding,
     error,
     pycompat,
     util,
 )
-from .utils import (
-    stringutil,
-)
+from .utils import stringutil
+
 
 def _typename(o):
-    return pycompat.sysbytes(type(o).__name__).lstrip('_')
+    return pycompat.sysbytes(type(o).__name__).lstrip(b'_')
+
 
 class abstractsmartset(object):
-
     def __nonzero__(self):
         """True if the smartset is not empty"""
         raise NotImplementedError()
@@ -64,7 +64,7 @@ class abstractsmartset(object):
             for v in self.fastasc():
                 break
             else:
-                raise ValueError('arg is an empty sequence')
+                raise ValueError(b'arg is an empty sequence')
         self.min = lambda: v
         return v
 
@@ -76,7 +76,7 @@ class abstractsmartset(object):
             for v in self.fastdesc():
                 break
             else:
-                raise ValueError('arg is an empty sequence')
+                raise ValueError(b'arg is an empty sequence')
         self.max = lambda: v
         return v
 
@@ -125,8 +125,9 @@ class abstractsmartset(object):
 
         This is part of the mandatory API for smartset."""
         c = other.__contains__
-        return self.filter(lambda r: not c(r), condrepr=('<not %r>', other),
-                           cache=False)
+        return self.filter(
+            lambda r: not c(r), condrepr=(b'<not %r>', other), cache=False
+        )
 
     def filter(self, condition, condrepr=None, cache=True):
         """Returns this smartset filtered by condition as a new smartset.
@@ -137,14 +138,14 @@ class abstractsmartset(object):
 
         This is part of the mandatory API for smartset."""
         # builtin cannot be cached. but do not needs to
-        if cache and util.safehasattr(condition, '__code__'):
+        if cache and util.safehasattr(condition, b'__code__'):
             condition = util.cachefunc(condition)
         return filteredset(self, condition, condrepr)
 
     def slice(self, start, stop):
         """Return new smartset that contains selected elements from this set"""
         if start < 0 or stop < 0:
-            raise error.ProgrammingError('negative index not allowed')
+            raise error.ProgrammingError(b'negative index not allowed')
         return self._slice(start, stop)
 
     def _slice(self, start, stop):
@@ -161,7 +162,8 @@ class abstractsmartset(object):
             if y is None:
                 break
             ys.append(y)
-        return baseset(ys, datarepr=('slice=%d:%d %r', start, stop, self))
+        return baseset(ys, datarepr=(b'slice=%d:%d %r', start, stop, self))
+
 
 class baseset(abstractsmartset):
     """Basic data structure that represents a revset and contains the basic
@@ -222,6 +224,7 @@ class baseset(abstractsmartset):
     >>> rs._istopo
     True
     """
+
     def __init__(self, data=(), datarepr=None, istopo=False):
         """
         datarepr: a tuple of (format, obj, ...), a function or an object that
@@ -342,20 +345,25 @@ class baseset(abstractsmartset):
 
     def _fastsetop(self, other, op):
         # try to use native set operations as fast paths
-        if (type(other) is baseset and r'_set' in other.__dict__ and r'_set' in
-            self.__dict__ and self._ascending is not None):
-            s = baseset(data=getattr(self._set, op)(other._set),
-                        istopo=self._istopo)
+        if (
+            type(other) is baseset
+            and r'_set' in other.__dict__
+            and r'_set' in self.__dict__
+            and self._ascending is not None
+        ):
+            s = baseset(
+                data=getattr(self._set, op)(other._set), istopo=self._istopo
+            )
             s._ascending = self._ascending
         else:
             s = getattr(super(baseset, self), op)(other)
         return s
 
     def __and__(self, other):
-        return self._fastsetop(other, '__and__')
+        return self._fastsetop(other, b'__and__')
 
     def __sub__(self, other):
-        return self._fastsetop(other, '__sub__')
+        return self._fastsetop(other, b'__sub__')
 
     def _slice(self, start, stop):
         # creating new list should be generally cheaper than iterating items
@@ -371,7 +379,7 @@ class baseset(abstractsmartset):
 
     @encoding.strmethod
     def __repr__(self):
-        d = {None: '', False: '-', True: '+'}[self._ascending]
+        d = {None: b'', False: b'-', True: b'+'}[self._ascending]
         s = stringutil.buildrepr(self._datarepr)
         if not s:
             l = self._list
@@ -381,13 +389,15 @@ class baseset(abstractsmartset):
             if self._ascending is not None:
                 l = self._asclist
             s = pycompat.byterepr(l)
-        return '<%s%s %s>' % (_typename(self), d, s)
+        return b'<%s%s %s>' % (_typename(self), d, s)
+
 
 class filteredset(abstractsmartset):
     """Duck type for baseset class which iterates lazily over the revisions in
     the subset and contains a function which tests for membership in the
     revset
     """
+
     def __init__(self, subset, condition=lambda x: True, condrepr=None):
         """
         condition: a function that decide whether a revision in the subset
@@ -427,10 +437,12 @@ class filteredset(abstractsmartset):
 
     def __nonzero__(self):
         fast = None
-        candidates = [self.fastasc if self.isascending() else None,
-                      self.fastdesc if self.isdescending() else None,
-                      self.fastasc,
-                      self.fastdesc]
+        candidates = [
+            self.fastasc if self.isascending() else None,
+            self.fastdesc if self.isdescending() else None,
+            self.fastasc,
+            self.fastdesc,
+        ]
         for candidate in candidates:
             if candidate is not None:
                 fast = candidate
@@ -484,7 +496,7 @@ class filteredset(abstractsmartset):
         if it is not None:
             for x in it():
                 return x
-            return None #empty case
+            return None  # empty case
         else:
             x = None
             for x in self:
@@ -497,7 +509,8 @@ class filteredset(abstractsmartset):
         s = stringutil.buildrepr(self._condrepr)
         if s:
             xs.append(s)
-        return '<%s %s>' % (_typename(self), ', '.join(xs))
+        return b'<%s %s>' % (_typename(self), b', '.join(xs))
+
 
 def _iterordered(ascending, iter1, iter2):
     """produce an ordered iteration from two iterators with the same order
@@ -535,6 +548,7 @@ def _iterordered(ascending, iter1, iter2):
         for val in it:
             yield val
 
+
 class addset(abstractsmartset):
     """Represent the addition of two sets
 
@@ -606,6 +620,7 @@ class addset(abstractsmartset):
     >>> [x for x in rs]
     [5, 4, 3, 2, 0]
     """
+
     def __init__(self, revs1, revs2, ascending=None):
         self._r1 = revs1
         self._r2 = revs2
@@ -641,6 +656,7 @@ class addset(abstractsmartset):
         if self._ascending is None:
             if self._genlist:
                 return iter(self._genlist)
+
             def arbitraryordergen():
                 for r in self._r1:
                     yield r
@@ -648,13 +664,14 @@ class addset(abstractsmartset):
                 for r in self._r2:
                     if not inr1(r):
                         yield r
+
             return arbitraryordergen()
         # try to use our own fast iterator if it exists
         self._trysetasclist()
         if self._ascending:
-            attr = 'fastasc'
+            attr = b'fastasc'
         else:
-            attr = 'fastdesc'
+            attr = b'fastdesc'
         it = getattr(self, attr)
         if it is not None:
             return it()
@@ -744,8 +761,9 @@ class addset(abstractsmartset):
 
     @encoding.strmethod
     def __repr__(self):
-        d = {None: '', False: '-', True: '+'}[self._ascending]
-        return '<%s%s %r, %r>' % (_typename(self), d, self._r1, self._r2)
+        d = {None: b'', False: b'-', True: b'+'}[self._ascending]
+        return b'<%s%s %r, %r>' % (_typename(self), d, self._r1, self._r2)
+
 
 class generatorset(abstractsmartset):
     """Wrap a generator for lazy iteration
@@ -760,6 +778,7 @@ class generatorset(abstractsmartset):
     >>> xs.last()  # cached
     4
     """
+
     def __new__(cls, gen, iterasc=None):
         if iterasc is None:
             typ = cls
@@ -830,7 +849,8 @@ class generatorset(abstractsmartset):
         # iteration.
         genlist = self._genlist
         nextgen = self._consumegen()
-        _len, _next = len, next # cache global lookup
+        _len, _next = len, next  # cache global lookup
+
         def gen():
             i = 0
             while True:
@@ -842,6 +862,7 @@ class generatorset(abstractsmartset):
                     except StopIteration:
                         return
                 i += 1
+
         return gen()
 
     def _consumegen(self):
@@ -908,8 +929,9 @@ class generatorset(abstractsmartset):
 
     @encoding.strmethod
     def __repr__(self):
-        d = {False: '-', True: '+'}[self._ascending]
-        return '<%s%s>' % (_typename(self), d)
+        d = {False: b'-', True: b'+'}[self._ascending]
+        return b'<%s%s>' % (_typename(self), d)
+
 
 class _generatorsetasc(generatorset):
     """Special case of generatorset optimized for ascending generators."""
@@ -930,6 +952,7 @@ class _generatorsetasc(generatorset):
         self._cache[x] = False
         return False
 
+
 class _generatorsetdesc(generatorset):
     """Special case of generatorset optimized for descending generators."""
 
@@ -949,6 +972,7 @@ class _generatorsetdesc(generatorset):
         self._cache[x] = False
         return False
 
+
 def spanset(repo, start=0, end=None):
     """Create a spanset that represents a range of repository revisions
 
@@ -964,6 +988,7 @@ def spanset(repo, start=0, end=None):
         start, end = end + 1, start + 1
     return _spanset(start, end, ascending, repo.changelog.filteredrevs)
 
+
 class _spanset(abstractsmartset):
     """Duck type for baseset class which represents a range of revisions and
     can work lazily and without having all the range in memory
@@ -974,6 +999,7 @@ class _spanset(abstractsmartset):
     - revision filtered with this repoview will be skipped.
 
     """
+
     def __init__(self, start, end, ascending, hiddenrevs):
         self._start = start
         self._end = end
@@ -1018,8 +1044,9 @@ class _spanset(abstractsmartset):
 
     def __contains__(self, rev):
         hidden = self._hiddenrevs
-        return ((self._start <= rev < self._end)
-                and not (hidden and rev in hidden))
+        return (self._start <= rev < self._end) and not (
+            hidden and rev in hidden
+        )
 
     def __nonzero__(self):
         for r in self:
@@ -1078,8 +1105,9 @@ class _spanset(abstractsmartset):
 
     @encoding.strmethod
     def __repr__(self):
-        d = {False: '-', True: '+'}[self._ascending]
-        return '<%s%s %d:%d>' % (_typename(self), d, self._start, self._end)
+        d = {False: b'-', True: b'+'}[self._ascending]
+        return b'<%s%s %d:%d>' % (_typename(self), d, self._start, self._end)
+
 
 class fullreposet(_spanset):
     """a set containing all revisions in the repo
@@ -1089,8 +1117,9 @@ class fullreposet(_spanset):
     """
 
     def __init__(self, repo):
-        super(fullreposet, self).__init__(0, len(repo), True,
-                                          repo.changelog.filteredrevs)
+        super(fullreposet, self).__init__(
+            0, len(repo), True, repo.changelog.filteredrevs
+        )
 
     def __and__(self, other):
         """As self contains the whole repo, all of the other set should also be
@@ -1099,7 +1128,7 @@ class fullreposet(_spanset):
         This boldly assumes the other contains valid revs only.
         """
         # other not a smartset, make is so
-        if not util.safehasattr(other, 'isascending'):
+        if not util.safehasattr(other, b'isascending'):
             # filter out hidden revision
             # (this boldly assumes all smartset are pure)
             #
