@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 ## statprof.py
 ## Copyright (C) 2012 Bryan O'Sullivan <bos@serpentine.com>
 ## Copyright (C) 2011 Alex Fraser <alex at phatcore dot com>
@@ -112,10 +111,10 @@ import json
 import os
 import signal
 import sys
-import tempfile
 import threading
 import time
 
+from .pycompat import open
 from . import (
     encoding,
     pycompat,
@@ -124,41 +123,52 @@ from . import (
 defaultdict = collections.defaultdict
 contextmanager = contextlib.contextmanager
 
-__all__ = ['start', 'stop', 'reset', 'display', 'profile']
+__all__ = [b'start', b'stop', b'reset', b'display', b'profile']
 
-skips = set(["util.py:check", "extensions.py:closure",
-             "color.py:colorcmd", "dispatch.py:checkargs",
-             "dispatch.py:<lambda>", "dispatch.py:_runcatch",
-             "dispatch.py:_dispatch", "dispatch.py:_runcommand",
-             "pager.py:pagecmd", "dispatch.py:run",
-             "dispatch.py:dispatch", "dispatch.py:runcommand",
-             "hg.py:<module>", "evolve.py:warnobserrors",
-         ])
+skips = {
+    r"util.py:check",
+    r"extensions.py:closure",
+    r"color.py:colorcmd",
+    r"dispatch.py:checkargs",
+    r"dispatch.py:<lambda>",
+    r"dispatch.py:_runcatch",
+    r"dispatch.py:_dispatch",
+    r"dispatch.py:_runcommand",
+    r"pager.py:pagecmd",
+    r"dispatch.py:run",
+    r"dispatch.py:dispatch",
+    r"dispatch.py:runcommand",
+    r"hg.py:<module>",
+    r"evolve.py:warnobserrors",
+}
 
 ###########################################################################
 ## Utils
 
+
 def clock():
     times = os.times()
-    return times[0] + times[1]
+    return (times[0] + times[1], times[4])
 
 
 ###########################################################################
 ## Collection data structures
 
+
 class ProfileState(object):
     def __init__(self, frequency=None):
         self.reset(frequency)
+        self.track = b'cpu'
 
     def reset(self, frequency=None):
         # total so far
-        self.accumulated_time = 0.0
+        self.accumulated_time = (0.0, 0.0)
         # start_time when timer is active
         self.last_start_time = None
         # a float
         if frequency:
             self.sample_interval = 1.0 / frequency
-        elif not hasattr(self, 'sample_interval'):
+        elif not pycompat.hasattr(self, 'sample_interval'):
             # default to 1000 Hz
             self.sample_interval = 1.0 / 1000.0
         else:
@@ -171,10 +181,24 @@ class ProfileState(object):
         self.samples = []
 
     def accumulate_time(self, stop_time):
-        self.accumulated_time += stop_time - self.last_start_time
+        increment = (
+            stop_time[0] - self.last_start_time[0],
+            stop_time[1] - self.last_start_time[1],
+        )
+        self.accumulated_time = (
+            self.accumulated_time[0] + increment[0],
+            self.accumulated_time[1] + increment[1],
+        )
 
     def seconds_per_sample(self):
-        return self.accumulated_time / len(self.samples)
+        return self.accumulated_time[self.timeidx] / len(self.samples)
+
+    @property
+    def timeidx(self):
+        if self.track == b'real':
+            return 1
+        return 0
+
 
 state = ProfileState()
 
@@ -182,18 +206,19 @@ state = ProfileState()
 class CodeSite(object):
     cache = {}
 
-    __slots__ = (u'path', u'lineno', u'function', u'source')
+    __slots__ = (r'path', r'lineno', r'function', r'source')
 
     def __init__(self, path, lineno, function):
+        assert isinstance(path, bytes)
         self.path = path
         self.lineno = lineno
+        assert isinstance(function, bytes)
         self.function = function
         self.source = None
 
     def __eq__(self, other):
         try:
-            return (self.lineno == other.lineno and
-                    self.path == other.path)
+            return self.lineno == other.lineno and self.path == other.path
         except:
             return False
 
@@ -213,31 +238,31 @@ class CodeSite(object):
     def getsource(self, length):
         if self.source is None:
             lineno = self.lineno - 1
-            fp = None
             try:
-                fp = open(self.path)
-                for i, line in enumerate(fp):
-                    if i == lineno:
-                        self.source = line.strip()
-                        break
+                with open(self.path, b'rb') as fp:
+                    for i, line in enumerate(fp):
+                        if i == lineno:
+                            self.source = line.strip()
+                            break
             except:
                 pass
-            finally:
-                if fp:
-                    fp.close()
             if self.source is None:
-                self.source = ''
+                self.source = b''
 
         source = self.source
         if len(source) > length:
-            source = source[:(length - 3)] + "..."
+            source = source[: (length - 3)] + b"..."
         return source
 
     def filename(self):
         return os.path.basename(self.path)
 
+    def skipname(self):
+        return r'%s:%s' % (self.filename(), self.function)
+
+
 class Sample(object):
-    __slots__ = (u'stack', u'time')
+    __slots__ = (r'stack', r'time')
 
     def __init__(self, stack, time):
         self.stack = stack
@@ -248,49 +273,67 @@ class Sample(object):
         stack = []
 
         while frame:
-            stack.append(CodeSite.get(frame.f_code.co_filename, frame.f_lineno,
-                                      frame.f_code.co_name))
+            stack.append(
+                CodeSite.get(
+                    pycompat.sysbytes(frame.f_code.co_filename),
+                    frame.f_lineno,
+                    pycompat.sysbytes(frame.f_code.co_name),
+                )
+            )
             frame = frame.f_back
 
         return Sample(stack, time)
 
+
 ###########################################################################
 ## SIGPROF handler
 
+
 def profile_signal_handler(signum, frame):
     if state.profile_level > 0:
         now = clock()
         state.accumulate_time(now)
 
-        state.samples.append(Sample.from_frame(frame, state.accumulated_time))
+        timestamp = state.accumulated_time[state.timeidx]
+        state.samples.append(Sample.from_frame(frame, timestamp))
 
-        signal.setitimer(signal.ITIMER_PROF,
-            state.sample_interval, 0.0)
+        signal.setitimer(signal.ITIMER_PROF, state.sample_interval, 0.0)
         state.last_start_time = now
 
+
 stopthread = threading.Event()
+
+
 def samplerthread(tid):
     while not stopthread.is_set():
         now = clock()
         state.accumulate_time(now)
 
         frame = sys._current_frames()[tid]
-        state.samples.append(Sample.from_frame(frame, state.accumulated_time))
+
+        timestamp = state.accumulated_time[state.timeidx]
+        state.samples.append(Sample.from_frame(frame, timestamp))
 
         state.last_start_time = now
         time.sleep(state.sample_interval)
 
     stopthread.clear()
 
+
 ###########################################################################
 ## Profiling API
 
+
 def is_active():
     return state.profile_level > 0
 
+
 lastmechanism = None
-def start(mechanism='thread'):
+
+
+def start(mechanism=b'thread', track=b'cpu'):
     '''Install the profiling signal handler, and start profiling.'''
+    state.track = track  # note: nesting different mode won't work
     state.profile_level += 1
     if state.profile_level == 1:
         state.last_start_time = clock()
@@ -300,73 +343,80 @@ def start(mechanism='thread'):
         global lastmechanism
         lastmechanism = mechanism
 
-        if mechanism == 'signal':
+        if mechanism == b'signal':
             signal.signal(signal.SIGPROF, profile_signal_handler)
-            signal.setitimer(signal.ITIMER_PROF,
-                rpt or state.sample_interval, 0.0)
-        elif mechanism == 'thread':
+            signal.setitimer(
+                signal.ITIMER_PROF, rpt or state.sample_interval, 0.0
+            )
+        elif mechanism == b'thread':
             frame = inspect.currentframe()
             tid = [k for k, f in sys._current_frames().items() if f == frame][0]
-            state.thread = threading.Thread(target=samplerthread,
-                                 args=(tid,), name="samplerthread")
+            state.thread = threading.Thread(
+                target=samplerthread, args=(tid,), name=b"samplerthread"
+            )
             state.thread.start()
 
+
 def stop():
     '''Stop profiling, and uninstall the profiling signal handler.'''
     state.profile_level -= 1
     if state.profile_level == 0:
-        if lastmechanism == 'signal':
+        if lastmechanism == b'signal':
             rpt = signal.setitimer(signal.ITIMER_PROF, 0.0, 0.0)
             signal.signal(signal.SIGPROF, signal.SIG_IGN)
             state.remaining_prof_time = rpt[0]
-        elif lastmechanism == 'thread':
+        elif lastmechanism == b'thread':
             stopthread.set()
             state.thread.join()
 
         state.accumulate_time(clock())
         state.last_start_time = None
-        statprofpath = encoding.environ.get('STATPROF_DEST')
+        statprofpath = encoding.environ.get(b'STATPROF_DEST')
         if statprofpath:
             save_data(statprofpath)
 
     return state
 
+
 def save_data(path):
-    with open(path, 'w+') as file:
-        file.write(str(state.accumulated_time) + '\n')
+    with open(path, b'w+') as file:
+        file.write(b"%f %f\n" % state.accumulated_time)
         for sample in state.samples:
-            time = str(sample.time)
+            time = sample.time
             stack = sample.stack
-            sites = ['\1'.join([s.path, str(s.lineno), s.function])
-                     for s in stack]
-            file.write(time + '\0' + '\0'.join(sites) + '\n')
+            sites = [
+                b'\1'.join([s.path, b'%d' % s.lineno, s.function])
+                for s in stack
+            ]
+            file.write(b"%d\0%s\n" % (time, b'\0'.join(sites)))
+
 
 def load_data(path):
-    lines = open(path, 'r').read().splitlines()
+    lines = open(path, b'rb').read().splitlines()
 
-    state.accumulated_time = float(lines[0])
+    state.accumulated_time = [float(value) for value in lines[0].split()]
     state.samples = []
     for line in lines[1:]:
-        parts = line.split('\0')
+        parts = line.split(b'\0')
         time = float(parts[0])
         rawsites = parts[1:]
         sites = []
         for rawsite in rawsites:
-            siteparts = rawsite.split('\1')
-            sites.append(CodeSite.get(siteparts[0], int(siteparts[1]),
-                        siteparts[2]))
+            siteparts = rawsite.split(b'\1')
+            sites.append(
+                CodeSite.get(siteparts[0], int(siteparts[1]), siteparts[2])
+            )
 
         state.samples.append(Sample(sites, time))
 
 
-
 def reset(frequency=None):
     '''Clear out the state of the profiler.  Do not call while the
     profiler is running.
 
     The optional frequency argument specifies the number of samples to
     collect per second.'''
-    assert state.profile_level == 0, "Can't reset() while statprof is running"
+    assert state.profile_level == 0, b"Can't reset() while statprof is running"
     CodeSite.cache.clear()
     state.reset(frequency)
 
@@ -384,6 +434,7 @@ def profile():
 ###########################################################################
 ## Reporting API
 
+
 class SiteStats(object):
     def __init__(self, site):
         self.site = site
@@ -424,7 +475,8 @@ class SiteStats(object):
                 if i == 0:
                     sitestat.addself()
 
-        return [s for s in stats.itervalues()]
+        return [s for s in pycompat.itervalues(stats)]
+
 
 class DisplayFormats:
     ByLine = 0
@@ -435,15 +487,18 @@ class DisplayFormats:
     Json = 5
     Chrome = 6
 
+
 def display(fp=None, format=3, data=None, **kwargs):
     '''Print statistics, either to stdout or the given file object.'''
-    data = data or state
+    if data is None:
+        data = state
 
     if fp is None:
         import sys
+
         fp = sys.stdout
     if len(data.samples) == 0:
-        print('No samples recorded.', file=fp)
+        fp.write(b'No samples recorded.\n')
         return
 
     if format == DisplayFormats.ByLine:
@@ -461,12 +516,13 @@ def display(fp=None, format=3, data=None, **kwargs):
     elif format == DisplayFormats.Chrome:
         write_to_chrome(data, fp, **kwargs)
     else:
-        raise Exception("Invalid display format")
+        raise Exception(b"Invalid display format")
 
     if format not in (DisplayFormats.Json, DisplayFormats.Chrome):
-        print('---', file=fp)
-        print('Sample count: %d' % len(data.samples), file=fp)
-        print('Total time: %f seconds' % data.accumulated_time, file=fp)
+        fp.write(b'---\n')
+        fp.write(b'Sample count: %d\n' % len(data.samples))
+        fp.write(b'Total time: %f seconds (%f wall)\n' % data.accumulated_time)
+
 
 def display_by_line(data, fp):
     '''Print the profiler data with each sample line represented
@@ -474,38 +530,51 @@ def display_by_line(data, fp):
     stats = SiteStats.buildstats(data.samples)
     stats.sort(reverse=True, key=lambda x: x.selfseconds())
 
-    print('%5.5s %10.10s   %7.7s  %-8.8s' %
-          ('%  ', 'cumulative', 'self', ''), file=fp)
-    print('%5.5s  %9.9s  %8.8s  %-8.8s' %
-          ("time", "seconds", "seconds", "name"), file=fp)
+    fp.write(
+        b'%5.5s %10.10s   %7.7s  %-8.8s\n'
+        % (b'%  ', b'cumulative', b'self', b'')
+    )
+    fp.write(
+        b'%5.5s  %9.9s  %8.8s  %-8.8s\n'
+        % (b"time", b"seconds", b"seconds", b"name")
+    )
 
     for stat in stats:
         site = stat.site
-        sitelabel = '%s:%d:%s' % (site.filename(), site.lineno, site.function)
-        print('%6.2f %9.2f %9.2f  %s' % (stat.selfpercent(),
-                                         stat.totalseconds(),
-                                         stat.selfseconds(),
-                                         sitelabel),
-              file=fp)
+        sitelabel = b'%s:%d:%s' % (site.filename(), site.lineno, site.function)
+        fp.write(
+            b'%6.2f %9.2f %9.2f  %s\n'
+            % (
+                stat.selfpercent(),
+                stat.totalseconds(),
+                stat.selfseconds(),
+                sitelabel,
+            )
+        )
+
 
 def display_by_method(data, fp):
     '''Print the profiler data with each sample function represented
     as one row in a table.  Important lines within that function are
     output as nested rows.  Sorted by self-time per line.'''
-    print('%5.5s %10.10s   %7.7s  %-8.8s' %
-          ('%  ', 'cumulative', 'self', ''), file=fp)
-    print('%5.5s  %9.9s  %8.8s  %-8.8s' %
-          ("time", "seconds", "seconds", "name"), file=fp)
+    fp.write(
+        b'%5.5s %10.10s   %7.7s  %-8.8s\n'
+        % (b'%  ', b'cumulative', b'self', b'')
+    )
+    fp.write(
+        b'%5.5s  %9.9s  %8.8s  %-8.8s\n'
+        % (b"time", b"seconds", b"seconds", b"name")
+    )
 
     stats = SiteStats.buildstats(data.samples)
 
     grouped = defaultdict(list)
     for stat in stats:
-        grouped[stat.site.filename() + ":" + stat.site.function].append(stat)
+        grouped[stat.site.filename() + b":" + stat.site.function].append(stat)
 
     # compute sums for each function
     functiondata = []
-    for fname, sitestats in grouped.iteritems():
+    for fname, sitestats in pycompat.iteritems(grouped):
         total_cum_sec = 0
         total_self_sec = 0
         total_percent = 0
@@ -514,11 +583,9 @@ def display_by_method(data, fp):
             total_self_sec += stat.selfseconds()
             total_percent += stat.selfpercent()
 
-        functiondata.append((fname,
-                             total_cum_sec,
-                             total_self_sec,
-                             total_percent,
-                             sitestats))
+        functiondata.append(
+            (fname, total_cum_sec, total_self_sec, total_percent, sitestats)
+        )
 
     # sort by total self sec
     functiondata.sort(reverse=True, key=lambda x: x[2])
@@ -526,28 +593,43 @@ def display_by_method(data, fp):
     for function in functiondata:
         if function[3] < 0.05:
             continue
-        print('%6.2f %9.2f %9.2f  %s' % (function[3], # total percent
-                                         function[1], # total cum sec
-                                         function[2], # total self sec
-                                         function[0]), # file:function
-              file=fp)
+        fp.write(
+            b'%6.2f %9.2f %9.2f  %s\n'
+            % (
+                function[3],  # total percent
+                function[1],  # total cum sec
+                function[2],  # total self sec
+                function[0],
+            )
+        )  # file:function
+
         function[4].sort(reverse=True, key=lambda i: i.selfseconds())
         for stat in function[4]:
             # only show line numbers for significant locations (>1% time spent)
             if stat.selfpercent() > 1:
                 source = stat.site.getsource(25)
-                stattuple = (stat.selfpercent(), stat.selfseconds(),
-                             stat.site.lineno, source)
+                if sys.version_info.major >= 3 and not isinstance(
+                    source, bytes
+                ):
+                    source = pycompat.bytestr(source)
+
+                stattuple = (
+                    stat.selfpercent(),
+                    stat.selfseconds(),
+                    stat.site.lineno,
+                    source,
+                )
+
+                fp.write(b'%33.0f%% %6.2f   line %d: %s\n' % stattuple)
 
-                print('%33.0f%% %6.2f   line %s: %s' % (stattuple), file=fp)
 
 def display_about_method(data, fp, function=None, **kwargs):
     if function is None:
-        raise Exception("Invalid function")
+        raise Exception(b"Invalid function")
 
     filename = None
-    if ':' in function:
-        filename, function = function.split(':')
+    if b':' in function:
+        filename, function = function.split(b':')
 
     relevant_samples = 0
     parents = {}
@@ -555,8 +637,9 @@ def display_about_method(data, fp, function=None, **kwargs):
 
     for sample in data.samples:
         for i, site in enumerate(sample.stack):
-            if site.function == function and (not filename
-                or site.filename() == filename):
+            if site.function == function and (
+                not filename or site.filename() == filename
+            ):
                 relevant_samples += 1
                 if i != len(sample.stack) - 1:
                     parent = sample.stack[i + 1]
@@ -570,17 +653,27 @@ def display_about_method(data, fp, function=None, **kwargs):
                 else:
                     children[site] = 1
 
-    parents = [(parent, count) for parent, count in parents.iteritems()]
+    parents = [(parent, count) for parent, count in pycompat.iteritems(parents)]
     parents.sort(reverse=True, key=lambda x: x[1])
     for parent, count in parents:
-        print('%6.2f%%   %s:%s   line %s: %s' %
-            (count / relevant_samples * 100, parent.filename(),
-            parent.function, parent.lineno, parent.getsource(50)), file=fp)
+        fp.write(
+            b'%6.2f%%   %s:%s   line %s: %s\n'
+            % (
+                count / relevant_samples * 100,
+                pycompat.fsencode(parent.filename()),
+                pycompat.sysbytes(parent.function),
+                parent.lineno,
+                pycompat.sysbytes(parent.getsource(50)),
+            )
+        )
 
     stats = SiteStats.buildstats(data.samples)
-    stats = [s for s in stats
-               if s.site.function == function and
-               (not filename or s.site.filename() == filename)]
+    stats = [
+        s
+        for s in stats
+        if s.site.function == function
+        and (not filename or s.site.filename() == filename)
+    ]
 
     total_cum_sec = 0
     total_self_sec = 0
@@ -592,23 +685,30 @@ def display_about_method(data, fp, function=None, **kwargs):
         total_self_percent += stat.selfpercent()
         total_cum_percent += stat.totalpercent()
 
-    print(
-        '\n    %s:%s    Total: %0.2fs (%0.2f%%)    Self: %0.2fs (%0.2f%%)\n' %
-        (
-        filename or '___',
-        function,
-        total_cum_sec,
-        total_cum_percent,
-        total_self_sec,
-        total_self_percent
-        ), file=fp)
+    fp.write(
+        b'\n    %s:%s    Total: %0.2fs (%0.2f%%)    Self: %0.2fs (%0.2f%%)\n\n'
+        % (
+            pycompat.sysbytes(filename or b'___'),
+            pycompat.sysbytes(function),
+            total_cum_sec,
+            total_cum_percent,
+            total_self_sec,
+            total_self_percent,
+        )
+    )
 
-    children = [(child, count) for child, count in children.iteritems()]
+    children = [(child, count) for child, count in pycompat.iteritems(children)]
     children.sort(reverse=True, key=lambda x: x[1])
     for child, count in children:
-        print('        %6.2f%%   line %s: %s' %
-              (count / relevant_samples * 100, child.lineno,
-               child.getsource(50)), file=fp)
+        fp.write(
+            b'        %6.2f%%   line %s: %s\n'
+            % (
+                count / relevant_samples * 100,
+                child.lineno,
+                pycompat.sysbytes(child.getsource(50)),
+            )
+        )
+
 
 def display_hotpath(data, fp, limit=0.05, **kwargs):
     class HotNode(object):
@@ -628,7 +728,7 @@ def display_hotpath(data, fp, limit=0.05, **kwargs):
             if len(stack) > 1:
                 i = 1
                 # Skip boiler plate parts of the stack
-                while i < len(stack) and '%s:%s' % (stack[i].filename(), stack[i].function) in skips:
+                while i < len(stack) and stack[i].skipname() in skips:
                     i += 1
                 if i < len(stack):
                     child.add(stack[i:], time)
@@ -638,38 +738,55 @@ def display_hotpath(data, fp, limit=0.05, **kwargs):
     for sample in data.samples:
         root.add(sample.stack[::-1], sample.time - lasttime)
         lasttime = sample.time
+    showtime = kwargs.get(r'showtime', True)
 
     def _write(node, depth, multiple_siblings):
         site = node.site
-        visiblechildren = [c for c in node.children.itervalues()
-                             if c.count >= (limit * root.count)]
+        visiblechildren = [
+            c
+            for c in pycompat.itervalues(node.children)
+            if c.count >= (limit * root.count)
+        ]
         if site:
             indent = depth * 2 - 1
-            filename = ''
-            function = ''
+            filename = b''
+            function = b''
             if len(node.children) > 0:
-                childsite = list(node.children.itervalues())[0].site
-                filename = (childsite.filename() + ':').ljust(15)
+                childsite = list(pycompat.itervalues(node.children))[0].site
+                filename = (childsite.filename() + b':').ljust(15)
                 function = childsite.function
 
             # lots of string formatting
-            listpattern = ''.ljust(indent) +\
-                          ('\\' if multiple_siblings else '|') +\
-                          ' %4.1f%%  %s %s'
-            liststring = listpattern % (node.count / root.count * 100,
-                                        filename, function)
-            codepattern = '%' + str(55 - len(liststring)) + 's %s:  %s'
-            codestring = codepattern % ('line', site.lineno, site.getsource(30))
+            listpattern = (
+                b''.ljust(indent)
+                + (b'\\' if multiple_siblings else b'|')
+                + b' %4.1f%%'
+                + (b' %5.2fs' % node.count if showtime else b'')
+                + b'  %s %s'
+            )
+            liststring = listpattern % (
+                node.count / root.count * 100,
+                filename,
+                function,
+            )
+            codepattern = b'%' + (b'%d' % (55 - len(liststring))) + b's %d:  %s'
+            codestring = codepattern % (
+                b'line',
+                site.lineno,
+                site.getsource(30),
+            )
 
             finalstring = liststring + codestring
-            childrensamples = sum([c.count for c in node.children.itervalues()])
+            childrensamples = sum(
+                [c.count for c in pycompat.itervalues(node.children)]
+            )
             # Make frames that performed more than 10% of the operation red
             if node.count - childrensamples > (0.1 * root.count):
-                finalstring = '\033[91m' + finalstring + '\033[0m'
+                finalstring = b'\033[91m' + finalstring + b'\033[0m'
             # Make frames that didn't actually perform work dark grey
             elif node.count - childrensamples == 0:
-                finalstring = '\033[90m' + finalstring + '\033[0m'
-            print(finalstring, file=fp)
+                finalstring = b'\033[90m' + finalstring + b'\033[0m'
+            fp.write(finalstring + b'\n')
 
         newdepth = depth
         if len(visiblechildren) > 1 or multiple_siblings:
@@ -682,41 +799,41 @@ def display_hotpath(data, fp, limit=0.05, **kwargs):
     if root.count > 0:
         _write(root, 0, False)
 
+
 def write_to_flame(data, fp, scriptpath=None, outputfile=None, **kwargs):
     if scriptpath is None:
-        scriptpath = encoding.environ['HOME'] + '/flamegraph.pl'
+        scriptpath = encoding.environ[b'HOME'] + b'/flamegraph.pl'
     if not os.path.exists(scriptpath):
-        print("error: missing %s" % scriptpath, file=fp)
-        print("get it here: https://github.com/brendangregg/FlameGraph",
-              file=fp)
+        fp.write(b'error: missing %s\n' % scriptpath)
+        fp.write(b'get it here: https://github.com/brendangregg/FlameGraph\n')
         return
 
-    fd, path = tempfile.mkstemp()
-
-    file = open(path, "w+")
-
     lines = {}
     for sample in data.samples:
         sites = [s.function for s in sample.stack]
         sites.reverse()
-        line = ';'.join(sites)
+        line = b';'.join(sites)
         if line in lines:
             lines[line] = lines[line] + 1
         else:
             lines[line] = 1
 
-    for line, count in lines.iteritems():
-        file.write("%s %s\n" % (line, count))
+    fd, path = pycompat.mkstemp()
 
-    file.close()
+    with open(path, b"w+") as file:
+        for line, count in pycompat.iteritems(lines):
+            file.write(b"%s %d\n" % (line, count))
 
     if outputfile is None:
-        outputfile = '~/flamegraph.svg'
+        outputfile = b'~/flamegraph.svg'
+
+    os.system(b"perl ~/flamegraph.pl %s > %s" % (path, outputfile))
+    fp.write(b'Written to %s\n' % outputfile)
 
-    os.system("perl ~/flamegraph.pl %s > %s" % (path, outputfile))
-    print("Written to %s" % outputfile, file=fp)
 
 _pathcache = {}
+
+
 def simplifypath(path):
     '''Attempt to make the path to a Python module easier to read by
     removing whatever part of the Python search path it was found
@@ -724,15 +841,16 @@ def simplifypath(path):
 
     if path in _pathcache:
         return _pathcache[path]
-    hgpath = pycompat.fsencode(encoding.__file__).rsplit(os.sep, 2)[0]
+    hgpath = encoding.__file__.rsplit(os.sep, 2)[0]
     for p in [hgpath] + sys.path:
         prefix = p + os.sep
         if path.startswith(prefix):
-            path = path[len(prefix):]
+            path = path[len(prefix) :]
             break
     _pathcache[path] = path
     return path
 
+
 def write_to_json(data, fp):
     samples = []
 
@@ -740,11 +858,22 @@ def write_to_json(data, fp):
         stack = []
 
         for frame in sample.stack:
-            stack.append((frame.path, frame.lineno, frame.function))
+            stack.append(
+                (
+                    pycompat.sysstr(frame.path),
+                    frame.lineno,
+                    pycompat.sysstr(frame.function),
+                )
+            )
 
         samples.append((sample.time, stack))
 
-    print(json.dumps(samples), file=fp)
+    data = json.dumps(samples)
+    if not isinstance(data, bytes):
+        data = data.encode('utf-8')
+
+    fp.write(data)
+
 
 def write_to_chrome(data, fp, minthreshold=0.005, maxthreshold=0.999):
     samples = []
@@ -755,7 +884,7 @@ def write_to_chrome(data, fp, minthreshold=0.005, maxthreshold=0.999):
     # representation to save space. It's fiddly but worth it.
     # We maintain a bijection between stack and ID.
     stack2id = {}
-    id2stack = [] # will eventually be rendered
+    id2stack = []  # will eventually be rendered
 
     def stackid(stack):
         if not stack:
@@ -765,14 +894,11 @@ def write_to_chrome(data, fp, minthreshold=0.005, maxthreshold=0.999):
         parent = stackid(stack[1:])
         myid = len(stack2id)
         stack2id[stack] = myid
-        id2stack.append(dict(category=stack[0][0], name='%s %s' % stack[0]))
+        id2stack.append(dict(category=stack[0][0], name=r'%s %s' % stack[0]))
         if parent is not None:
             id2stack[-1].update(parent=parent)
         return myid
 
-    def endswith(a, b):
-        return list(a)[-len(b):] == list(b)
-
     # The sampling profiler can sample multiple times without
     # advancing the clock, potentially causing the Chrome trace viewer
     # to render single-pixel columns that we cannot zoom in on.  We
@@ -803,8 +929,16 @@ def write_to_chrome(data, fp, minthreshold=0.005, maxthreshold=0.999):
         if minthreshold <= duration <= maxthreshold:
             # ensure no zero-duration events
             sampletime = max(oldtime + clamp, sample.time)
-            samples.append(dict(ph='E', name=oldfunc, cat=oldcat, sf=oldsid,
-                                ts=sampletime*1e6, pid=0))
+            samples.append(
+                dict(
+                    ph=r'E',
+                    name=oldfunc,
+                    cat=oldcat,
+                    sf=oldsid,
+                    ts=sampletime * 1e6,
+                    pid=0,
+                )
+            )
         else:
             blacklist.add(oldidx)
 
@@ -812,12 +946,16 @@ def write_to_chrome(data, fp, minthreshold=0.005, maxthreshold=0.999):
     # events given only stack snapshots.
 
     for sample in data.samples:
-        tos = sample.stack[0]
-        name = tos.function
-        path = simplifypath(tos.path)
-        category = '%s:%d' % (path, tos.lineno)
-        stack = tuple((('%s:%d' % (simplifypath(frame.path), frame.lineno),
-                        frame.function) for frame in sample.stack))
+        stack = tuple(
+            (
+                (
+                    r'%s:%d'
+                    % (simplifypath(pycompat.sysstr(frame.path)), frame.lineno),
+                    pycompat.sysstr(frame.function),
+                )
+                for frame in sample.stack
+            )
+        )
         qstack = collections.deque(stack)
         if laststack == qstack:
             continue
@@ -831,19 +969,35 @@ def write_to_chrome(data, fp, minthreshold=0.005, maxthreshold=0.999):
             laststack.appendleft(f)
             path, name = f
             sid = stackid(tuple(laststack))
-            samples.append(dict(ph='B', name=name, cat=path, ts=sample.time*1e6,
-                                sf=sid, pid=0))
+            samples.append(
+                dict(
+                    ph=r'B',
+                    name=name,
+                    cat=path,
+                    ts=sample.time * 1e6,
+                    sf=sid,
+                    pid=0,
+                )
+            )
         laststack = collections.deque(stack)
     while laststack:
         poplast()
-    events = [s[1] for s in enumerate(samples) if s[0] not in blacklist]
-    frames = collections.OrderedDict((str(k), v)
-                                     for (k,v) in enumerate(id2stack))
-    json.dump(dict(traceEvents=events, stackFrames=frames), fp, indent=1)
-    fp.write('\n')
+    events = [
+        sample for idx, sample in enumerate(samples) if idx not in blacklist
+    ]
+    frames = collections.OrderedDict(
+        (str(k), v) for (k, v) in enumerate(id2stack)
+    )
+    data = json.dumps(dict(traceEvents=events, stackFrames=frames), indent=1)
+    if not isinstance(data, bytes):
+        data = data.encode('utf-8')
+    fp.write(data)
+    fp.write(b'\n')
+
 
 def printusage():
-    print("""
+    print(
+        r"""
 The statprof command line allows you to inspect the last profile's results in
 the following forms:
 
@@ -860,7 +1014,9 @@ usage:
     flame [-s --script-path] [-o --output-file path]
         Writes out a flamegraph to output-file (defaults to ~/flamegraph.svg)
         Requires that ~/flamegraph.pl exist.
-        (Specify alternate script path with --script-path.)""")
+        (Specify alternate script path with --script-path.)"""
+    )
+
 
 def main(argv=None):
     if argv is None:
@@ -873,58 +1029,62 @@ def main(argv=None):
     displayargs = {}
 
     optstart = 2
-    displayargs['function'] = None
-    if argv[1] == 'hotpath':
-        displayargs['format'] = DisplayFormats.Hotpath
-    elif argv[1] == 'lines':
-        displayargs['format'] = DisplayFormats.ByLine
-    elif argv[1] == 'functions':
-        displayargs['format'] = DisplayFormats.ByMethod
-    elif argv[1] == 'function':
-        displayargs['format'] = DisplayFormats.AboutMethod
-        displayargs['function'] = argv[2]
+    displayargs[b'function'] = None
+    if argv[1] == r'hotpath':
+        displayargs[b'format'] = DisplayFormats.Hotpath
+    elif argv[1] == r'lines':
+        displayargs[b'format'] = DisplayFormats.ByLine
+    elif argv[1] == r'functions':
+        displayargs[b'format'] = DisplayFormats.ByMethod
+    elif argv[1] == r'function':
+        displayargs[b'format'] = DisplayFormats.AboutMethod
+        displayargs[b'function'] = argv[2]
         optstart = 3
-    elif argv[1] == 'flame':
-        displayargs['format'] = DisplayFormats.FlameGraph
+    elif argv[1] == r'flame':
+        displayargs[b'format'] = DisplayFormats.FlameGraph
     else:
         printusage()
         return 0
 
     # process options
     try:
-        opts, args = pycompat.getoptb(sys.argv[optstart:], "hl:f:o:p:",
-                                   ["help", "limit=", "file=", "output-file=", "script-path="])
+        opts, args = pycompat.getoptb(
+            sys.argv[optstart:],
+            b"hl:f:o:p:",
+            [b"help", b"limit=", b"file=", b"output-file=", b"script-path="],
+        )
     except getopt.error as msg:
         print(msg)
         printusage()
         return 2
 
-    displayargs['limit'] = 0.05
+    displayargs[b'limit'] = 0.05
     path = None
     for o, value in opts:
-        if o in ("-l", "--limit"):
-            displayargs['limit'] = float(value)
-        elif o in ("-f", "--file"):
+        if o in (r"-l", r"--limit"):
+            displayargs[b'limit'] = float(value)
+        elif o in (r"-f", r"--file"):
             path = value
-        elif o in ("-o", "--output-file"):
-            displayargs['outputfile'] = value
-        elif o in ("-p", "--script-path"):
-            displayargs['scriptpath'] = value
-        elif o in ("-h", "help"):
+        elif o in (r"-o", r"--output-file"):
+            displayargs[b'outputfile'] = value
+        elif o in (r"-p", r"--script-path"):
+            displayargs[b'scriptpath'] = value
+        elif o in (r"-h", r"help"):
             printusage()
             return 0
         else:
-            assert False, "unhandled option %s" % o
+            assert False, b"unhandled option %s" % o
 
     if not path:
-        print('must specify --file to load')
+        print(r'must specify --file to load')
         return 1
 
     load_data(path=path)
 
-    display(**displayargs)
+    display(**pycompat.strkwargs(displayargs))
 
     return 0
 
-if __name__ == "__main__":
+
+if __name__ == r"__main__":
     sys.exit(main())
